/*
 * Copyright (c) 2020, Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#define	TRUSTED_NV_CTR_ID	U(0)
#define	NON_TRUSTED_NV_CTR_ID	U(1)
#define	MAX_NV_CTR_IDS		U(2)
