/*
 * Copyright (c) 2020, The Linux Foundation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#ifndef QTI_RNG_IO_H
#define QTI_RNG_IO_H

#define SEC_PRNG_STATUS			0x00791004
#define SEC_PRNG_STATUS_DATA_AVAIL_BMSK	0x1
#define SEC_PRNG_DATA_OUT		0x00791000


#endif /* QTI_RNG_IO_H */

