/*
 * Copyright (c) 2016-2019, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef AEM_GENERIC_H
#define AEM_GENERIC_H

#include <lib/utils_def.h>

/* BASE AEM midr for revision 0 */
#define BASE_AEM_MIDR		U(0x410FD0F0)

#endif /* AEM_GENERIC_H */
