/*
 * Copyright (c) 2020, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef __PLAT_HELPERS_H__
#define __PLAT_HELPERS_H__

unsigned int plat_mediatek_calc_core_pos(u_register_t mpidr);

#endif /* __PLAT_HELPERS_H__ */
