/*
 * Copyright (c) 2017-2020, NVIDIA CORPORATION. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef TEGRA186_PRIVATE_H
#define TEGRA186_PRIVATE_H

uint64_t tegra186_get_mc_ctx_size(void);

#endif /* TEGRA186_PRIVATE_H */
