/*
 * Copyright (c) 2012-2017 Roberto E. Vargas Caballero
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
/*
 * Portions copyright (c) 2018, ARM Limited and Contributors.
 * All rights reserved.
 */

#ifndef LIMITS_H
#define LIMITS_H

#include <limits_.h>

#define CHAR_BIT   8
#define MB_LEN_MAX 1

#endif /* LIMITS_H */
