/*
 * Copyright (c) 2019, Arm Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef RPI3_RNG_H
#define RPI3_RNG_H

void rpi3_rng_read(void *buf, size_t len);

#endif
