/*
 * Copyright (c) 2018-2019, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef STDIO__H
#define STDIO__H

#ifndef SSIZET_
typedef int ssize_t;
#define SSIZET_
#endif

#endif /* STDIO__H */
