/*
 * Copyright (c) 2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PLAT_MACROS_S
#define PLAT_MACROS_S

/*
 * Print CCN registers
 */
	.macro plat_crash_print_regs
	.endm

#endif /* PLAT_MACROS_S */
