/*
 * Copyright (C) 2018 Marvell International Ltd.
 *
 * SPDX-License-Identifier:     BSD-3-Clause
 * https://spdx.org/licenses
 */

#ifndef AP807_INIT_CLOCKS_H
#define AP807_INIT_CLOCKS_H

void ap807_clocks_init(unsigned int freq_option);

#endif /* AP807_INIT_CLOCKS_H */

