/*
 * Copyright (c) 2020, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef tc0_bl31_common_platform_setup_PLAT_H
#define tc0_bl31_common_platform_setup_PLAT_H

void tc0_bl31_common_platform_setup(void);

#endif /* tc0_bl31_common_platform_setup_PLAT_H */
