/*
 * Copyright (c) 2020-2021, Renesas Electronics Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef RZG_QOS_INIT_H
#define RZG_QOS_INIT_H

void rzg_qos_init(void);
uint8_t get_boardcnf_phyvalid(void);

#endif /* RZG_QOS_INIT_H */
